// Procedural brick texture;  inspired by ch. 6 in the orange book
// GZ, 2014

varying vec4 LightIntensity;		// contains diffuse and specular terms, w/o object color
varying vec2 Pos_mc;				// the position of the vertex/fragment in model(!) coords


void main()
{
	LightIntensity = gl_LightSource[0].ambient;

	// first transform the normal into eye space and normalize the result
	vec3 normal = normalize(gl_NormalMatrix * gl_Normal);

	// now normalize the light's direction. Note that according to the OpenGL
	// specification, the light is stored in eye space. Also since we're 
	// talking about a directional light, the position field is actually direction.
	vec3 lightDir = normalize( gl_LightSource[0].position.xyz );

	float NdotL = max( dot(normal,lightDir), 0.0 );
	LightIntensity += NdotL * gl_LightSource[0].diffuse;

	if ( NdotL > 0.0 )
	{
		// per-vertex half-vector, assuming directional light
		vec4 v = gl_ModelViewMatrix * gl_Vertex;
		vec3 view = - normalize( vec3(v.xyz) );		// vector from v to eye(= 0,0,0)
		vec3 halfVec = normalize( lightDir + view );

		float NdotH = max( dot(normal,halfVec), 0.0 );
		if ( NdotH > 0.0 )
			LightIntensity += gl_LightSource[0].specular * 
                    			pow(NdotH, gl_FrontMaterial.shininess);
	}

	// Hand model coord of vertex to rasterizer and fragment shader
	Pos_mc = gl_Vertex.xy;
	
	gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex;
}
