//
// lighting4.vert
//

// Per-pixel lighting with a directional light

// The following variables are just to pass data from the vertex
// shader to the fragment shader; they do not need to be
// interpolated; but that way we can balance the workload between
// the two shaders
varying vec4 diffuse, ambient;
varying vec3 lightDir, halfVector;

// this is the only variable that really needs to be interpolated
varying vec3 normal;

void main()
{		
	// first transform the normal into eye space and normalize the
	// result
	normal = normalize(gl_NormalMatrix * gl_Normal);

	// transform light direction and vertex into camera space
	vec4 l = gl_ModelViewMatrix * gl_LightSource[0].position;
	vec4 v = gl_ModelViewMatrix * gl_Vertex;

	// now normalize the light's direction. Note that according to
	// the OpenGL specification, the light is stored in eye space.
	// Also since we're talking about a directional light, the
	// position field is actually direction.
	lightDir = normalize( l.xyz*v.w - v.xyz*l.w );

	// the per-vertex half-vector (assuming directional light)
	halfVector = l.xyz / l.w;
	vec4 view = -v;								// eye = 0
	view.xyz /= view.w;
	halfVector += view.xyz;						// h = l + view
	halfVector = normalize( halfVector );

	// Compute Kd * light
	diffuse = gl_FrontMaterial.diffuse * gl_LightSource[0].diffuse;

	// Compute Ka * ambient light
	ambient = gl_FrontMaterial.ambient * gl_LightSource[0].ambient;
	ambient += gl_LightModel.ambient * gl_FrontMaterial.ambient;
	
    gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex;
}
