// Procedural brick texture; inspired by ch. 6 in the orange book
// Includes noise
// GZ, 2014


uniform vec4 BrickColor, MortarColor;
uniform vec2 BrickStepSize, BrickPercent;

uniform float noise_spatial_freq;	// spatial frequency of the noise
uniform float noise_amplitude;		// use starting values: freq = 5  amp = 0.2
uniform float noise_time_shift;		// doesn't make sense here, but for demo purposes


varying vec4 LightIntensity;		// contains diffuse and specular terms, w/o object color
varying vec2 Pos_mc;				// the position of the vertex/fragment in model(!) coords


void main()
{
	vec2 pos = Pos_mc / BrickStepSize;		// now pos = 0.0,..,1.0 for one brick

	// are we in an odd brick row?
	if ( mod(abs(pos.y), 2.0) > 1.0 )
		pos.x += 0.5;

	pos = fract( pos );					// = rel. position within brick

	// determine color
	vec4 color;
	vec4 dirt;
	if ( any( greaterThan(pos, BrickPercent) ) )
	{
		color = MortarColor;
		// add some "dirt"
		float c = noise_amplitude * noise1( noise_spatial_freq * Pos_mc + 
									  vec2(noise_time_shift) );
		dirt = vec4( c, c, c, 1.0 );
	}
	else
	{
		color = BrickColor;
		// add some "dirt"
		vec3 c = noise_amplitude * noise3( noise_spatial_freq * Pos_mc + 
									  vec2(noise_time_shift) );
		dirt = vec4( c, 1.0 );	
	}
	color += dirt;

	color *= LightIntensity;
	gl_FragColor = color;
}
