#version 330 core

// 
// lighting3.vert
//
// Run with ShaderMaker
//
// This set of shaders (lighting[1-3].{vert,frag}) shows 
// per-pixel lighting step-by-step
//
// Author: GZ, 2007, 2012, 2019


// Should we output the view vector, have it interpolated by the rasterizer,
// and calculate the half vector in the fragment shader?

// Parameters pre-defined on host side (as uniforms)
uniform mat4 viewMatrix;      // model-to-world-to-camera matrix of the model
uniform mat4 projection;      // projection matrix (depends on aspect ratio of window)

// Attributes per vertex
in vec3 vPosition;            // xyz position of the vertex
in vec3 vNormal;              // xyz normal of the vertex

// Attributes to be calculated by this shader, for the following fragment shader
out vec3 half_vector;         // half vector (to be interpolated by rasterizer)
out vec3 normal;              // will be the normal per fragment
flat out vec3 light_dir;      // pass-through; we can use 'flat', because it's a direction


void main()
{
	vec3 light_direction = vec3(-1, 1, 1);       // we define the direction directly in camera coord

	// Transform the normal into eye space and normalize the result
	// We can use the viewing matrix here, because ShaderMaker uses only rigid transformations
	normal = ( viewMatrix * vec4(vNormal, 0.0) ).xyz;

	// transform the vertex into camera space
	vec4 vpos_cam_space = viewMatrix * vec4(vPosition, 1.0);

	// compute the half vector   (note: view vector = eye - vertex = 0 - vpos_cam_space)
	vec3 view_vector = normalize( vpos_cam_space.xyz / vpos_cam_space.w );
	light_direction = normalize( light_direction );
	half_vector = light_direction - view_vector;
	light_dir = light_direction;

	gl_Position = projection * vpos_cam_space;
}

