#version 330 core

// Ein Shader, der sog. "Toon Shading" macht, d.h., die Objekte so
// darstellt, dass sie aehnlich wie in einem Cartoon aussehen (hence the name)
//
// Dieser Shader kommt auch mit positional light sources zurecht, daher
// der etwas hoehere Aufwand.
//
// GZ 2011
// PD 2017


uniform vec4 LightSource;

// Mit dieser uniform-Variable koennen wir aus dem Shader Editor die Grundfarbe
// des Toon-Shadings setzen (wir ignorieren naemlich die Objekt-/Vertex-Farbe, die von 
// der Applikation heringereicht wird)
uniform vec3 Color;
out vec4 FragColor;

// Kommt vom Rasterizer bzw. Vertex-Shader
in vec3 fNormal;
in vec3 surf_pos;


void main()
{
	// Berechne Vektor von unserer Position in Weltkoord. zur Lichtquelle
	// Der folgende Ausdruck ist (im Prinzip) aequivalent zu 
	// normalize( LightSource.xyz / LightSource[0].w - position ),
	// funktioniert aber auch dann, wenn Lichtquelle = Richtungsvektor (oder position)
	vec3 l = normalize( LightSource.xyz - surf_pos * LightSource.w  );

	// Normiere Normale (kann durch Rasterizer Laenge ungleich 1 bekommen)
	vec3 n = normalize( fNormal );

	// Setze Helligkeit fuer dieses Pixel abhaengig vom Winkel zwischen
	// Lichtvektor und Normale ...
	float intensity = dot( l, n );

	vec4 c;
	if (intensity > 0.95)							// ... aber in diskreten Levels
		c = vec4( Color,1.0 );
	else if (intensity > 0.7)
		c = vec4( 0.7*Color, 1.0 );
	else if (intensity > 0.4)
		c = vec4( 0.5*Color, 1.0 );
	else
		c = vec4( 0.3*Color, 1.0 );

	FragColor = c;
}
